-- Last Edit:     #Date#
-- Last Revision: #Rev#
-- Last Edit By:  #Author#
----------------------------------------------------------------------------------
-- This file handles all the needed info to controll the Objective frame corectly.
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Local values
----------------------------------------------------------------------------------
local freeContentHeadlines, HeadlineCount = {}, 0
local freeContentImages, ImageCount = {}, 0
local freeContentItem, ItemCount = {}, 0
local freeContentObjective, ObjectiveCount = {}, 0
local freeContentTexts, TextCount = {}, 0
local freeContentTextHighlites, TextHighliteCount = {}, 0
local freeTables = {}
local L = ProGuideLocals
local objCount = 0
local offset = 0
local x, y = 0, 0
local RefreshFrames = {}
local tempFrames = {}

-- Content types allowed
local typeDef = {"all", "headline", "image", "item", "objective", "text", "highlite",}

----------------------------------------------------------------------------------
-- local functions
----------------------------------------------------------------------------------
-- Get new/old content headline
local function GetContentHeadline(self)
  if #freeContentHeadlines == 0 then HeadlineCount = HeadlineCount + 1 end
  return tremove(freeContentHeadlines) or CreateFrame("Frame", "ProGuideHeadline" .. HeadlineCount, self.ScrollFrame.Content, "ProGuideContentHeadlineTemplate")
end

-- Get new/old content image
local function GetContentImage(self)
  if #freeContentImages == 0 then ImageCount = ImageCount + 1 end
  return tremove(freeContentImages) or CreateFrame("Frame", "ProGuideImage" .. ImageCount, self.ScrollFrame.Content, "ProGuideContentImageTemplate")
end

-- Get new/old content item
local function GetContentItem(self)
  if #freeContentItem == 0 then ItemCount = ItemCount + 1 end
  return tremove(freeContentItem) or CreateFrame("Frame", "ProGuideItem" .. ItemCount, self.ScrollFrame.Content, "ProGuideContentItemTemplate")
end

-- Get new/old content objective
local function GetContentObjective(self)
  if #freeContentObjective == 0 then ObjectiveCount = ObjectiveCount + 1 end
  return tremove(freeContentObjective) or CreateFrame("Frame", "ProGuideObjective" .. ObjectiveCount, self.ScrollFrame.Content, "ProGuideContentObjectiveTemplate")
end

-- Get new/old content text
local function GetContentText(self)
  if #freeContentTexts == 0 then TextCount = TextCount + 1 end
  return tremove(freeContentTexts) or CreateFrame("Frame", "ProGuideText" .. TextCount, self.ScrollFrame.Content, "ProGuideContentTextTemplate")
end

-- Get new/old content text
local function ContentTextHighlites(self)
  if #freeContentTextHighlites == 0 then TextHighliteCount = TextHighliteCount + 1 end
  return tremove(freeContentTextHighlites) or CreateFrame("Frame", "ProGuideTextHighlite" .. TextHighliteCount, self.ScrollFrame.Content, "ProGuideContentTextHighliteTemplate")
end

-- Get new/old table
local function GetTable()
  return tremove(freeTables) or {}
end

-- Rearange Content
local function RearangeContent(self)
  for i = 1, #self.Content, 1 do
    if self.Content[i]["type"] == "item" then
      if self.Content[i-1] then
        if self.Content[i-1]["type"] == "item" then
          offset = 5
        else
          offset = 0
        end
      else
        offset = -2
      end
    else
      offset = -5
    end
    if i == 1 then
      self.Content[i].frame:SetPoint("TOP", self.ScrollFrame.Content, "TOP", 0, offset)
    else
      self.Content[i].frame:SetPoint("TOP", self.Content[i - 1].frame, "BOTTOM", 0, offset)
    end
  end
end

-- Racalculate content height
local function RecalculateContentHeight(self)
  self.ScrollFrame.Content:SetHeight(5)
  for i = 1, #self.Content, 1 do
    if self.Content[i]["type"] == "item" then
      if self.Content[i-1] then
        if self.Content[i-1]["type"] == "item" then
          offset = -5
        else
          offset = 0
        end
      else
        offset = 2
      end
    else
      offset = 5
    end
    self.ScrollFrame.Content:SetHeight(self.ScrollFrame.Content:GetHeight() + self.Content[i].frame:GetHeight() + offset)
  end
end

-- Recycle old content
local function RecycleContent(t, Rtype)
  if Rtype and (not tContains(typeDef, Rtype)) then return end

  if (not Rtype) or Rtype == "all" then Rtype = t.type end

  if Rtype == "headline" then
    t.frame:Hide()
    t.frame.Headline:SetText(L["Unused"])
    tinsert(freeContentHeadlines, t.frame)

  elseif Rtype == "image" then
    t.frame:Hide()
    tinsert(freeContentImages, t.frame)

  elseif Rtype == "item" then
    t.frame:Hide()
    t.frame.ItemButton.IconTexture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
    t.frame.ButtonText:Hide()
    t.frame.Text:Hide()
    t.frame.Text:SetText(L["Getting Item Info"])
    t.frame.ItemButton.IconGlow:Hide()
    t.frame.Counter:Hide()
    t.frame.DoneMarker:Hide()
    tinsert(freeContentItem, t.frame)

  elseif Rtype == "objective" then
    t.frame:Hide()
    t.frame.Text:SetText(L["Unused"])
    t.frame.DoneMarker:Hide()
    t.frame.MapButton:Hide()
    if t.frame.MapButton.map then t.frame.MapButton.map = nil end
    if t.frame.MapButton.tooltip then t.frame.MapButton.tooltip = nil end
    if t.frame.MapButton.file then t.frame.MapButton.file = nil end
    if t.frame.MapButton.zone then t.frame.MapButton.zone = nil end
    tinsert(freeContentObjective, t.frame)

  elseif Rtype == "text" then
    t.frame:Hide()
    t.frame.Text:SetText(L["Unused"])
    tinsert(freeContentTexts, t.frame)

  elseif Rtype == "highlite" then
    t.frame:Hide()
    t.frame.TextHighlite:SetText(L["Unused"])
    tinsert(freeContentTextHighlites, t.frame)
  end
end

-- Wait for return value
local function WaitForItemInfo(self)
  tinsert(RefreshFrames, self)
end

-- Wait for return value
local function WaitForItemInfo(self)
  tinsert(RefreshFrames, self)
end

-- Add recieved item info
local function RecievedInfo(self)
  if self.type == "item" then
    local name, link, quality, _, _, _, _, _, _, texture, _ = GetItemInfo(self.infoId)
    if name then
      if self.npcId and ProGuide.Npc[self.npcId] then
        self.frame.ButtonText.Text:SetText(name)
        self.frame.ButtonText.tooltip = L["BuyFrom"]:format(ProGuide.Npc[self.npcId].Name)
        self.frame.ButtonText.npc = self.npcId
        self.frame.ButtonText:Show()
        self.frame.Text:Hide()
      else
        self.frame.Text:SetText(name)
        self.frame.Text:Show()
        self.frame.ButtonText:Hide()
      end
      self.frame.ItemButton.link = link
      self.frame.ItemButton.IconTexture:SetTexture(texture)
      self.frame.ItemButton.IconGlow:SetVertexColor(GetItemQualityColor(quality))
      self.frame.ItemButton.IconGlow:Show()
      if self.checked then
        self.frame.DoneMarker:Show()
      elseif self.number and type(self.number) == "number" then
        self.frame.Counter:SetText(L["%d of %d"]:format( GetItemCount(infoId, true), self.number))
        self.frame.Counter:Show()
      end
    else
      WaitForItemInfo(self)
    end
  end
end

-- Recycle old table
local function RecycleTable(t)
  for key in pairs(t) do
    t[key] = nil
  end
  tinsert(freeTables, t)
end

----------------------------------------------------------------------------------
-- Update Item Info by using Event: GET_ITEM_INFO_RECEIVED
----------------------------------------------------------------------------------
local infoCatchFrame = CreateFrame("Frame")
infoCatchFrame:SetScript("OnEvent", function(self, event)
  if #RefreshFrames > 0 then
    -- Copy anc clear RefreshFrame
    tempFrames = RefreshFrames
    RefreshFrames = {}
    for i = #tempFrames, 1, -1 do
      RecievedInfo(tempFrames[i])
    end
    tempFrames = {}
  end
end)
infoCatchFrame:RegisterEvent("GET_ITEM_INFO_RECEIVED")

----------------------------------------------------------------------------------
-- Global functions
----------------------------------------------------------------------------------
-- Create a new objective frame
function ProGuideInitialice()
  -- Calculate new obj count
  objCount = objCount + 1

  -- Create a new Objective frame
  local obj = CreateFrame("Frame", "ProGuideObj" .. tostring(objCount), UIParent, "ProGuideObjFramesTemplate")
  -- we only needs the first one to be moveable
  if objCount > 1 then
    obj:SetPoint("TOP", "ProGuideObj" .. tostring(objCount - 1), "BOTTOM" ,0 , 10)
    obj:SetMovable(false)
    obj:RegisterForDrag("None")
  end
  obj.ButtonGuides:SetText(L["Guides"])
  obj.ButtonBuyList:SetText(L["Buy List"])

  obj.Map = CreateFrame("Button", "ProGuideMap" .. tostring(objCount), obj, "ProGuideObjMapFramesTemplate")
  obj.Map:SetPoint("RIGHT", obj, "LEFT")

  obj.Content = GetTable()

  -- Add default headline
  ProGuideEditHeadline(obj, L["No Guide Loaded"])
  ProGuideEditSubHeadline(obj, "Skill 200-300")
  ProGuideEditTextLevel(obj, "Skill Level 245")
  ProGuideAddContentHeadline(obj, "Content Headline 01")
  ProGuideAddContentObjective(obj, "With Map", nil, "A_Herbalism_ElwynnForest.tga", "Herbalism Grinding Zone", "Elwynn Forest")
  ProGuideAddContentObjective(obj, "With Map but no zone", nil, "A_Mining_ElwynnForest.tga", "Mining Grinding Zone", nil)
  ProGuideAddContentObjective(obj, "Without Map")
  ProGuideAddContentObjective(obj, "Compleated", true)
  ProGuideAddContentTextHighlite(obj, "Get items listed below")
  ProGuideAddContentItem(obj, 29739, 100, nil, 5499)
  ProGuideAddContentItem(obj, 49482, 1)
  ProGuideAddContentText(obj, "Content text that holds large textareas with multiline of some bla bla bla bla bla bla bla bla bla bla bla bla bla bla bla")
  ProGuideAddContentImage(obj, "Interface\\AddOns\\TourGuide-Tradeskill\\Maps\\H_Herbalism_Durotar.tga", 384, 256, nil, 0.75, nil, nil, nil)

  -- return the new Objective frame
  return obj
end

-- Edit Headline
function ProGuideEditHeadline(self, text)
  if (not self) or type(self) ~= "table" then return end
  if not text then text = "" end
  self.TextHeadline:SetText(text)
end

-- Edit Sub Headline
function ProGuideEditSubHeadline(self, text)
  if (not self) or type(self) ~= "table" then return end
  if not text then text = "" end
  self.TextSubHeadline:SetText(text)
end

-- Edit Level
function ProGuideEditTextLevel(self, text)
  if (not self) or type(self) ~= "table" then return end
  if not text then text = "" end
  self.TextLevel:SetText(text)
end

-- Add Content Headline
function ProGuideAddContentHeadline(self, text, id)
  if (not self) or type(self) ~= "table" then return end
  if (not text) or text == "" then return end
  if (not id) or type(id) ~= "number" then id = #self.Content + 1 end

  tinsert(self.Content, {["id"] = id, ["frame"] = GetContentHeadline(self), ["type"] = "headline",})
  self.Content[id].frame:SetParent(self.ScrollFrame.Content)
  self.Content[id].frame.Headline:SetText(text)
  self.Content[id].frame:SetHeight(self.Content[id].frame.Headline:GetStringHeight() + 2)
  self.Content[id].frame:Show()
  RearangeContent(self)
  RecalculateContentHeight(self)
end

-- Add Content Image
function ProGuideAddContentImage(self, file, width, height, left, right, top, bottom, id)
  if (not self) or type(self) ~= "table" then return end
  if (not file) or file == "" then return end
  if (not id) or type(id) ~= "number" then id = #self.Content + 1 end
  if (not width) or (not height) or type(width) ~= "number" or type(height) ~= "number" then return end
  if width <= 270 then
    x = width
    y = height
  else
    x = 270
    y = (270 / width) * height
  end

  tinsert(self.Content, {["id"] = id, ["frame"] = GetContentImage(self), ["type"] = "image",})
  self.Content[id].frame:SetParent(self.ScrollFrame.Content)
  self.Content[id].frame:SetSize(x, y)
  self.Content[id].frame.Image:SetTexture(file)
  self.Content[id].frame.Image:SetSize(x, y)
  self.Content[id].frame.Image:SetTexCoord(left or 0, right or 1, top or 0, bottom or 1)
  self.Content[id].frame:Show()
  RearangeContent(self)
  RecalculateContentHeight(self)
end

-- Add Content Item
function ProGuideAddContentItem(self, infoId, number, checked, npcId, id)
  if (not self) or type(self) ~= "table" then return end
  if (not infoId) or type(infoId) ~= "number" then return end
  if (not id) or type(id) ~= "number" then id = #self.Content + 1 end

  tinsert(self.Content, {["id"] = id, ["frame"] = GetContentItem(self), ["infoId"] = infoId, ["type"] = "item",})
  self.Content[id].frame:SetParent(self.ScrollFrame.Content)

  local name, link, quality, _, _, _, _, _, _, texture, _ = GetItemInfo(infoId)
  if name then
    if npcId and ProGuide.Npc[npcId] then
      self.Content[id].frame.ButtonText.Text:SetText(name)
      self.Content[id].frame.ButtonText.tooltip = L["Can be bought from %s\n|cFF008000Click to show waypoint.|r"]:format(ProGuide.Npc[npcId].Name)
      self.Content[id].frame.ButtonText.npc = npcId
      self.Content[id].frame.ButtonText:Show()
      self.Content[id].frame.Text:Hide()
    else
      self.Content[id].frame.Text:SetText(name)
      self.Content[id].frame.Text:Show()
      self.Content[id].frame.ButtonText:Hide()
    end
    self.Content[id].frame.ItemButton.link = link
    self.Content[id].frame.ItemButton.IconTexture:SetTexture(texture)
    self.Content[id].frame.ItemButton.IconGlow:SetVertexColor(GetItemQualityColor(quality))
    self.Content[id].frame.ItemButton.IconGlow:Show()
    if checked then
      self.Content[id].frame.DoneMarker:Show()
    elseif number and type(number) == "number" then
      self.Content[id].frame.Counter:SetText(L["%d of %d"]:format( GetItemCount(infoId, true), number))
      self.Content[id].frame.Counter:Show()
    end
  else
    self.Content[id].frame.Text:SetText(L["Getting Item Info"])
    self.Content[id].frame.Text:Show()
    self.Content[id].frame.ItemButton.IconTexture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
    self.Content[id].number = number
    self.Content[id].checked = checked
    self.Content[id].npcId = npcId
    WaitForItemInfo(self.Content[id])
  end

  self.Content[id].frame:Show()
  RearangeContent(self)
  RecalculateContentHeight(self)
end

-- Add Content Objective
function ProGuideAddContentObjective(self, text, checked, map, tooltip, zone, id)
  if (not self) or type(self) ~= "table" then return end
  if (not text) or text == "" then return end
  if (not id) or type(id) ~= "number" then id = #self.Content + 1 end

  tinsert(self.Content, {["id"] = id, ["frame"] = GetContentObjective(self), ["type"] = "objective",})
  self.Content[id].frame:SetParent(self.ScrollFrame.Content)
  self.Content[id].frame.Text:SetText(text)
  if checked then
    self.Content[id].frame.DoneMarker:Show()
  elseif map then
    self.Content[id].frame.MapButton.map = self.Map
    self.Content[id].frame.MapButton.file = map
    self.Content[id].frame.MapButton.tooltip = tooltip
    self.Content[id].frame.MapButton.zone = zone
    self.Content[id].frame.MapButton:Show()
  end
  self.Content[id].frame:Show()
  RearangeContent(self)
  RecalculateContentHeight(self)
end

-- Add Content Text
function ProGuideAddContentText(self, text, id)
  if (not self) or type(self) ~= "table" then return end
  if (not text) or text == "" then return end
  if (not id) or type(id) ~= "number" then id = #self.Content + 1 end

  tinsert(self.Content, {["id"] = id, ["frame"] = GetContentText(self), ["type"] = "text",})
  self.Content[id].frame:SetParent(self.ScrollFrame.Content)
  self.Content[id].frame.Text:SetText(text)
  self.Content[id].frame:SetHeight(self.Content[id].frame.Text:GetStringHeight() + 2)
  self.Content[id].frame:Show()
  RearangeContent(self)
  RecalculateContentHeight(self)
end

-- Add Content Text Highlite
function ProGuideAddContentTextHighlite(self, text, id)
  if (not self) or type(self) ~= "table" then return end
  if (not text) or text == "" then return end
  if (not id) or type(id) ~= "number" then id = #self.Content + 1 end

  tinsert(self.Content, {["id"] = id, ["frame"] = ContentTextHighlites(self), ["type"] = "highlite",})
  self.Content[id].frame:SetParent(self.ScrollFrame.Content)
  self.Content[id].frame.TextHighlite:SetText(text)
  self.Content[id].frame:SetHeight(self.Content[id].frame.TextHighlite:GetStringHeight() + 2)
  self.Content[id].frame:Show()
  RearangeContent(self)
  RecalculateContentHeight(self)
end

-- Remove Content
function ProGuideRemoveContent(self, Rtype, pos, id)
  if (not self) or type(self) ~= "table" then return end
  if #self.Content <= 0 then return end
  if ((not Rtype) or (not tContains(typeDef, Rtype))) and
     ((not pos) or type(pos) ~= "number") and
     ((not id) or type(id) ~= "number") 
  then
    return
  end

  if Rtype and pos then
    if self.Content[pos].type == Rtype then
      RecycleContent(self.Content[pos], Rtype)
      tremove(self.Content, pos)
      RearangeContent(self)
      RecalculateContentHeight(self)
    end
  elseif Rtype and id then
    for i = #self.Content, 1, -1 do
      if self.Content[i].id == id then
        if self.Content[i].type == Rtype then
          RecycleContent(self.Content[i], Rtype)
          tremove(self.Content, i)
          RearangeContent(self)
          RecalculateContentHeight(self)
        end
        return
      end
    end
  elseif pos then
    RecycleContent(self.Content[pos])
    tremove(self.Content, pos)
    RearangeContent(self)
    RecalculateContentHeight(self)
  elseif id then
    for i = #self.Content, 1, -1 do
      if self.Content[i].id == id then
        RecycleContent(self.Content[i], Rtype)
        tremove(self.Content, i)
        RearangeContent(self)
        RecalculateContentHeight(self)
        return
      end
    end
  else
    for i = #self.Content, 1, -1 do
      if Rtype == "all" or Rtype == self.Content[i].type then
        RecycleContent(self.Content[i], Rtype)
        tremove(self.Content, i)
      end
    end
    RearangeContent(self)
    RecalculateContentHeight(self)
  end
end